% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/resistance.R
\name{resistance}
\alias{resistance}
\alias{susceptibility}
\alias{n_rsi}
\alias{rsi}
\title{Calculate resistance of isolates}
\usage{
resistance(ab, include_I = TRUE, minimum = 30, as_percent = FALSE)

susceptibility(ab1, ab2 = NULL, include_I = FALSE, minimum = 30,
  as_percent = FALSE)

n_rsi(ab1, ab2 = NULL)

rsi(ab1, ab2 = NULL, interpretation = "IR", minimum = 30,
  as_percent = FALSE)
}
\arguments{
\item{ab, ab1, ab2}{vector of antibiotic interpretations, they will be transformed internally with \code{\link{as.rsi}}}

\item{include_I}{logical to indicate whether antimicrobial interpretations of "I" should be included}

\item{minimum}{minimal amount of available isolates. Any number lower than \code{minimum} will return \code{NA}.}

\item{as_percent}{logical to indicate whether the output must be returned as percent (text), will else be a double}

\item{interpretation}{antimicrobial interpretation}
}
\value{
Double or, when \code{as_percent = TRUE}, a character.
}
\description{
These functions can be used to calculate the (co-)resistance of microbial isolates (i.e. percentage S, SI, I, IR or R). All functions can be used in \code{dplyr}s \code{\link[dplyr]{summarise}} and support grouped variables, see \emph{Examples}.
}
\details{
\strong{Remember that you should filter your table to let it contain only first isolates!} Use \code{\link{first_isolate}} to determine them in your data set.

All return values are calculated using hybrid evaluation (i.e. using C++), which makes these functions 60-65 times faster than in \code{AMR} v0.2.0 and below. The \code{rsi} function is available for backwards compatibility and deprecated. It now uses the \code{resistance} and \code{susceptibility} functions internally, based on the \code{interpretation} parameter.
\if{html}{
  \cr
  To calculate the probability (\emph{p}) of susceptibility of one antibiotic, we use this formula:
  \out{<div style="text-align: center">}\figure{mono_therapy.png}\out{</div>}
  To calculate the probability (\emph{p}) of susceptibility of more antibiotics (i.e. combination therapy), we need to check whether one of them has a susceptible result (as numerator) and count all cases where all antibiotics were tested (as denominator). \cr
  \cr
  For two antibiotics:
  \out{<div style="text-align: center">}\figure{combi_therapy_2.png}\out{</div>}
  \cr
  Theoretically for three antibiotics:
  \out{<div style="text-align: center">}\figure{combi_therapy_3.png}\out{</div>}
}
}
\examples{
library(dplyr)

septic_patients \%>\%
  group_by(hospital_id) \%>\%
  summarise(p = susceptibility(cipr),
            n = n_rsi(cipr)) # n_rsi works like n_distinct in dplyr

septic_patients \%>\%
  group_by(hospital_id) \%>\%
  summarise(cipro_p = susceptibility(cipr, as_percent = TRUE),
            cipro_n = n_rsi(cipr),
            genta_p = susceptibility(gent, as_percent = TRUE),
            genta_n = n_rsi(gent),
            combination_p = susceptibility(cipr, gent, as_percent = TRUE),
            combination_n = n_rsi(cipr, gent))


# Calculate resistance
resistance(septic_patients$amox)
rsi(septic_patients$amox, interpretation = "IR") # deprecated

# Or susceptibility
susceptibility(septic_patients$amox)
rsi(septic_patients$amox, interpretation = "S") # deprecated


# Calculate co-resistance between amoxicillin/clav acid and gentamicin,
# so we can see that combination therapy does a lot more than mono therapy:
susceptibility(septic_patients$amcl) # p = 67.8\%
n_rsi(septic_patients$amcl)          # n = 1641

susceptibility(septic_patients$gent) # p = 69.1\%
n_rsi(septic_patients$gent)          # n = 1863

with(septic_patients,
     susceptibility(amcl, gent))     # p = 90.6\%
with(septic_patients,
     n_rsi(amcl, gent))              # n = 1580

\dontrun{
# calculate current empiric combination therapy of Helicobacter gastritis:
my_table \%>\%
  filter(first_isolate == TRUE,
         genus == "Helicobacter") \%>\%
  summarise(p = susceptibility(amox, metr), # amoxicillin with metronidazole
            n = n_rsi(amox, metr))
}
}
\keyword{antibiotics}
\keyword{isolate}
\keyword{isolates}
\keyword{resistance}
\keyword{rsi_df}
\keyword{susceptibility}
